<?php

namespace Schaffen\ProductionOrder\ProductionOrder\Repositories;

use DateTime;
use Freelabois\LaravelQuickstart\Extendables\Repository;
use Schaffen\ProductionOrder\ProductionOrder\Models\CompositeProductHistoric;
use Schaffen\ProductionOrder\ProductionOrder\Models\CompositeProduct;
use Schaffen\ProductionOrder\ProductionOrder\Models\ProductionOrder;
use Schaffen\Core\Product\Models\ProductVarietyPack;
use Illuminate\Support\Carbon;

class CompositeProductHistoricRepository extends Repository 
{
    protected $model = CompositeProductHistoric::class;

    protected $searchableFields = [
        ["field" => "pch_fk_pve_codigo_acabado"],
        ["field" => "pch_fk_op_codigo"],
        [
            "name" => "operation_in",
            "operator" => "in",
            "field" => "pch_operacao"
        ]
    ];

    const STORE = 'CADASTRO_PCP';
    const UPDATE = 'EDICAO_PCP';
    const DESTROY_PCP = 'EXCLUSAO_PCP';
    const STORE_OP = 'CADASTRO_ORDEM_PRODUCAO';
    const UPDATE_OP = 'EDICAO_ORDEM_PRODUCAO';
    const UPDATE_OP_STATUS = 'ALTERACAO_STATUS_ORDEM_PRODUCAO';
    const FINALIZATION_OP = 'FINALIZACAO_ORDEM_PRODUCAO';
    const DESTROY_OP = 'EXCLUSAO_OP';

    public function storeOrUpdatePCP(CompositeProduct $composite_product, ProductVarietyPack $final_product, $operation) {

        $description = implode('<br>', [
            "Produto acabado: {$final_product->pve_codigo} - {$final_product->product_complete_name}",
            "Matéria-prima: {$composite_product->product_variety_pack->pve_codigo} - {$composite_product->product_variety_pack->product_complete_name}", 
            "Unidade de medida: {$composite_product->product_variety_pack->measurement_unit->unm_descricao}", 
            "Quantidade: " . number_format($composite_product->pcp_quantidade, 2, ',', '.'),
            "Valor unitário: " . number_format($composite_product->pcp_valor_unitario, 2, ',', '.'), 
            $composite_product->pcp_controla_estoque ? "Controla estoque: Sim" : "Controla estoque: Não",
            "Custo de produção previsto:" . number_format($composite_product->pcp_total, 2, ',', '.')
        ]);

        $values = [
            'pch_fk_pve_codigo_acabado' => $composite_product->pcp_fk_pve_codigo_acabado,
            'pch_operacao' => $operation,
            'pch_descricao' => $description
        ];

        $this->storeOrUpdate($values);
    }

    public function storeOrUpdateOP(ProductionOrder $production_order, $composition_products = null, $operation, $status = null) {

        $date = new DateTime("{$production_order->op_data_agendamento} {$production_order->op_hora_agendamento}");

        $items = null;
        if ($composition_products) {
            $items = array_reduce($composition_products, function($text, $product) {
                $item = "{$product->feedstock->pve_codigo} - {$product->feedstock->product_complete_name}<br>";
                return $text . $item;
            }, 'Itens:<br>');
        }

        $description = implode('<br>', array_filter([
            "Ordem de produção: {$production_order->op_codigo}", 
            "Produto acabado: {$production_order->finished_product->pve_codigo} – {$production_order->finished_product->product_complete_name}", 
            "Data de agendamento: " . $date->format('d/m/Y'), 
            "Hora de agendamento: " . $date->format('H:i:s'), 
            "Quantidade prevista: {$production_order->op_quantidade}", 
            !empty($production_order->op_observacao) ? "Observação: {$production_order->op_observacao}" : "Observação: - ",
            $items
        ]));

        $values = [
            'pch_fk_pve_codigo_acabado' => $production_order->finished_product->pve_codigo,
            'pch_operacao' => $operation,
            'pch_fk_op_codigo' => $production_order->op_codigo,
            'pch_descricao' => $description
        ];

        $this->storeOrUpdate($values);
    }

    private function buildSatausDescription($status) {
        
        $description = $status->st_descricao;
        $control_warehouse = boolval($status->st_controla_estoque) ? 'Sim' : 'Não';
        $warehouse_origin = boolval($status->st_controla_estoque) ? $status->warehouse_inventory_origin()->first()->loe_descricao : null;
        $warehouse_destiny = boolval($status->st_controla_estoque) ? $status->warehouse_inventory_destiny()->first()->loe_descricao : null;

        return implode(', ',  array_filter([
            $description,
            "Controla estoque: {$control_warehouse}",
            $warehouse_origin ? "Local de estoque origem: {$warehouse_origin}" : null,
            $warehouse_destiny ? "Local de estoque destino: {$warehouse_destiny}" : null
        ]));
    }


    public function changeStatusOP(ProductionOrder $production_order, $info) {

        $old_status = $this->buildSatausDescription($info['old_status']);
        $new_status = $this->buildSatausDescription($info['new_status']);
 
        $description = implode('<br>', array_filter([
            "Ordem de produção: {$production_order->op_codigo}", 
            "Status alterado de: {$old_status}",
            "para: {$new_status}"
        ]));

        $operation = $info['new_status']->isFinal() ? CompositeProductHistoricRepository::FINALIZATION_OP : CompositeProductHistoricRepository::UPDATE_OP_STATUS;

        $values = [
            'pch_fk_pve_codigo_acabado' => $production_order->finished_product->pve_codigo,
            'pch_operacao' => $operation,
            'pch_fk_op_codigo' => $production_order->op_codigo,
            'pch_descricao' => $description
        ];


        $this->storeOrUpdate($values);
    }
}