<?php


namespace Schaffen\ProductionOrder\ProductionOrder\Repositories;


use Schaffen\ProductionOrder\ProductionOrder\Models\ProductionOrder;
use Freelabois\LaravelQuickstart\Extendables\Repository;

class ProductionOrderRepository extends Repository
{
    protected $model = ProductionOrder::class;
    protected $searchableFields = [
        ['field' => 'op_fk_pve_final'],
        ['field' => 'op_codigo'],
    ];

    public function findManyOpen(array $filters = [], array $with = [])
    {
        $this->applyFilters($filters);
        $query = $this->newQuery();
        $query->with($with);

        $query->whereHas('status', function ($query) {
            $query->where('st_controla_estoque', 1);
        });

        if (!empty($this->filters)) {
            $this->applyCustomFilters();
            $this->injectFiltersOnQuery();
        }

        $this->group();
        $this->order();
        $this->returnable = $query->get();

        return $this->present(true);
    }

    public function injectFiltersOnQuery() {
        $initial_date = NULL;
        $final_date = NULL;
        $st_codigo = NULL;

        extract($this->filters);

        $this->query->when($initial_date, function($query) use ($initial_date) {
            return $query->where('op_dta_agendamento', '>=', $initial_date);
        });

        $this->query->when($final_date, function($query) use ($final_date) {
            return $query->where('op_dta_agendamento', '<=', $final_date);
        });

        $this->query->when($st_codigo, function($query) use ($st_codigo) {
            return $query->whereHas('status', function($query) use ($st_codigo) {
                return $query->where('st_codigo', $st_codigo);
            });
        });

        parent::injectFiltersOnQuery();
    }
}
